%% RHPCancellation
% This file generates a step response for a feedback system where the
% controller and plant have a pole and zero that cancel in the RHP plane.
% For this example, the complementary sensitivity (r to y) is stable
% but the transfer function from r to u is unstable.

%% Plant and Controller

G = tf([1 -1],[1 3]);
K = tf(1,[1 -1]);

%% Feedback System Transfer Functions

% Complementary Sensitivity (r to y)
% Note G*K has a common pole and zero at s=+1 in the RHP. The minreal
% function is used to obtain a minimal realization.
L = minreal(G*K);
T = feedback(L,1);  

disp('T is stable since all poles are in the LHP')
pole(T)

% Transfer function from r to u
Tr2u = feedback(K,G);

disp('Tr2u is unstable since it has a pole in the RHP')
pole(Tr2u)


%% Unit Step Responses

% Final Simulation Time
Tf = 3;

% Output, y
[y,ty] = step(T,Tf);

figure(1); clf;
plot(ty,y,'b'); 
xlabel('Time, $t$');
ylabel('Output, $y(t)$');
axis([0 Tf 0 0.27])
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));
if exist('garyfyFigure','file') && exist('matlab2tikz','file');
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure    
    matlab2tikz('RHPCancel_Y.tikz','width','\figurewidth',...
         'parseStrings',false,'extraAxisOptions',...
         'ylabel near ticks, xlabel near ticks');
end


% Controller Command, u
[u,tu] = step(Tr2u,Tf);

figure(2); clf;
plot(tu,u,'b'); 
xlabel('Time, $t$');
ylabel('Control, $u(t)$');
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));
%set(gca,'YtickLabelMode','manual')
if exist('garyfyFigure','file') && exist('matlab2tikz','file');
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure    
    matlab2tikz('RHPCancel_U.tikz','width','\figurewidth',...
         'parseStrings',false,'extraAxisOptions',...
         'ylabel near ticks, xlabel near ticks');
end

% Step Reference Command
tr = [0; tu];
Nt = numel(tr);
r = [0; ones(Nt-1,1)];

figure(3); clf;
plot(tr,r,'b'); 
xlabel('Time, $t$');
ylabel('Reference, $r(t)$');
grid on;
axis([-0.05 Tf 0 1.1])
set(gca, 'LooseInset', get(gca, 'TightInset'));
if exist('garyfyFigure','file') && exist('matlab2tikz','file');
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure    
    matlab2tikz('RHPCancel_R.tikz','width','\figurewidth',...
         'parseStrings',false,'extraAxisOptions',...
         'ylabel near ticks, xlabel near ticks');
end

