%% Loopshaping Requirements
% This file generates figures to show representive requirements (bounds)
% on S, T, and the loop L=GK.

%% Frequency vector
Nw = 1000;
w = logspace(-3,3,Nw)';

%% Representative Loop L=GK
G = tf(1,[1 0.7 0.01]); 
K = 1;
L = G*K;

%% Bode Magnitudes of S, T, and L
% Also compute frequencies for bandwidths: |L|=0dB, |S|=-3dB, |T|=-3dB
Lmag = bode(L,w);
Lmag = Lmag(:);
LmagdB = 20*log10(Lmag);
[~,Lidx] = min(abs(LmagdB));
wL = w(Lidx);

T = feedback(L,1);
Tmag = bode(T,w);
Tmag = Tmag(:);
TmagdB = 20*log10(Tmag);
[~,Tidx] = min(abs(TmagdB+3));
wT = w(Tidx);


S = feedback(1,L);
Smag = bode(S,w);
Smag = Smag(:);
SmagdB = 20*log10(Smag);
[~,Sidx] = min(abs(SmagdB+3));
wS = w(Sidx);

%% Requirements Bounds on S, T, and L

% Upper Bound on |S|
eps1 = 0.05;
w1 = 0.02;
Smax = 2.5;

idx1 = (w <= w1);
Sub = zeros(Nw,1);
Sub(idx1) = eps1;
Sub(~idx1) = Smax;
SubdB = 20*log10(Sub);

% Upper Bound on |T|
eps2 = 0.05;
w2 = 10;
Tmax = 2.5;


idx2 = (w >= w2);
Tub = zeros(Nw,1);
Tub(idx2) = eps2;
Tub(~idx2) = Tmax;
TubdB = 20*log10(Tub);

% Lower and Upper Bounds on |L|
Llb = zeros(Nw,1);
Llb(idx1) = 1/eps1;
LlbdB = 20*log10(Llb);

Lub = zeros(Nw,1);
Lub(idx2) = eps2;
LubdB = 20*log10(Lub);


%% Plot S, T, and L (Representative Loop and Bounds)

% Limits for axes
xmin = w(1);
xmax = w(end);
ymin = -50;
ymax = 15;

% Plot |S|
figure(1)
subplot(2,1,1)
semilogx(w,SmagdB,'k-.',w,SubdB,'b');
hold on;
lhS = semilogx([wS wS xmin],[ymin -3 -3],'k');
hold off;
axis([xmin xmax ymin ymax]);
phS = patch([xmin w1 w1 xmax xmax xmin xmin],...
    [SubdB(1) SubdB(1) 8 8 ymax ymax SubdB(1)],'b');
set(phS,'FaceAlpha',0.05,'LineStyle','none');
set(gca,'YTick',[SubdB(1) -3 SubdB(end)],'LineWidth',2, ...
        'YTickLabel',{'\epsilon_1', -3, 8});
set(gca,'XTick',[w1 wS],'LineWidth',2, ...
        'XTickLabel',{'\omega_1','\omega_S'})
ylabel('|S|, dB');


% Plot |T|
subplot(2,1,2)
semilogx(w,TmagdB,'k-.',w,TubdB,'r');
hold on;
lhT = semilogx([wT wT xmin],[ymin -3 -3],'k');
hold off;
axis([xmin xmax ymin ymax]);
phT = patch([xmin w2 w2 xmax xmax xmin xmin],...
    [8 8 TubdB(end) TubdB(end) ymax ymax 8],'r');
set(phT,'FaceAlpha',0.05,'LineStyle','none');
set(gca,'YTick',[TubdB(end) -3 TubdB(1)],'LineWidth',2, ...
        'YTickLabel',{'\epsilon_2', -3, 8});
set(gca,'XTick',[wT w2],'LineWidth',2, ...
        'XTickLabel',{'\omega_T','\omega_2'})
xlabel('Frequency, rad/sec');
ylabel('|T|, dB');


% Plot |L|
yminL = Lub(end)-40;
ymaxL = Llb(1) +40;

figure(2)
semilogx(w,LmagdB,'k-.');
hold on;
lhL = semilogx([wL wL xmin],[ymin 0 0],'k');
hold off;
axis([xmin xmax yminL ymaxL]);
phLlb = patch([w(1) w1 w1 w(1)],[yminL yminL LlbdB(1) LlbdB(1)],'b');
phLub = patch([w(end) w2 w2 w(end)],[ymaxL ymaxL LubdB(end) LubdB(end)],'r');
set(phLlb,'FaceAlpha',0.05,'EdgeColor','b','LineWidth',3);
set(phLub,'FaceAlpha',0.05,'EdgeColor','r','LineWidth',3);
set(gca,'YTick',[TubdB(end) 0 -SubdB(1)],'LineWidth',2, ...
        'YTickLabel',{'\epsilon_2', 0, '1/\epsilon_1'});
set(gca,'XTick',[w1 wL w2],'LineWidth',2, ...
        'XTickLabel',{'\omega_1','\omega_c','\omega_2'})
xlabel('Frequency, rad/sec');
ylabel('|L|=|GK|, dB');

% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    set([lhS; lhT],'LineWidth',1);
    myprint(1,'STbounds')
    
    figure(2)
    garyfyFigure
    set(lhL,'LineWidth',1);
    myprint(2,'Lbounds')    
end




